/*
   Copyright (C) 2008 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "messageroutewidget.h"

#include <QGridLayout>
#include <QLabel>
#include <QNetworkAccessManager>
#include <QNetworkReply>

#include <KDebug>
#include <KDialog>
#include <KLocale>
#include <KProgressDialog>

#include <marble/MarbleWidget.h>
#include <marble/GeoDataPlacemark.h>

using namespace Mailody;
using namespace Marble;


MessageRouteWidget::MessageRouteWidget( QWidget* parent )
        : KDialog( parent )
{
    setButtons( KDialog::Ok );
    setCaption( i18n( "Message route" ) );

    m_marble = new Marble::MarbleWidget( this );
    m_marble->setMapThemeId( "earth/srtm/srtm.dgml" );

    setMainWidget( m_marble );

    setMinimumSize( 400,400 );
    KConfigGroup config = KGlobal::config()->group( "MessageRoute" );
    restoreDialogSize( config );
}

void MessageRouteWidget::accept()
{
    KConfigGroup config = KGlobal::config()->group( "MessageRoute" );
    saveDialogSize( config );
    config.sync();
    done( QDialog::Accepted );
}

void MessageRouteWidget::setIPs( const QStringList& ips )
{
    m_ips = ips;
    m_progress = new KProgressDialog( this, i18n( "Fetching locations..." ),
                                      i18n( "Fetching the location, please wait..." ) );
    m_progress->progressBar()->setMaximum( m_ips.count() );

    foreach( const QString& ip, m_ips ) {
        getIPMethod1( ip );
    }
}

void MessageRouteWidget::getIPMethod1( const QString& ip )
{
    QNetworkAccessManager *manager = new QNetworkAccessManager( this );
    connect( manager, SIGNAL( finished( QNetworkReply* ) ),
             this, SLOT( slotReplyFinished1( QNetworkReply* ) ) );
    manager->get( QNetworkRequest( "http://api.hostip.info/get_html.php?position=true&ip=" + ip ) );
}

void MessageRouteWidget::getIPMethod2( const QString& ip )
{
    QNetworkAccessManager *manager = new QNetworkAccessManager( this );
    connect( manager, SIGNAL( finished( QNetworkReply* ) ),
             this, SLOT( slotReplyFinished2( QNetworkReply* ) ) );
    manager->get( QNetworkRequest( "http://netgeo.caida.org/perl/netgeo.cgi?target=" + ip ) );
}

void MessageRouteWidget::slotReplyFinished1( QNetworkReply *reply )
{
    QString tempLat, tempLong, tempCity, ip;
    const QList<QByteArray> data = reply->readAll().split( '\n' );
    ip = reply->url().toString().mid( 53 );
    foreach( const QByteArray& line, data ) {
        if ( line.startsWith( "Latitude: " ) )
            tempLat = line.mid( 10 );
        else if ( line.startsWith( "Longitude: " ) )
            tempLong = line.mid( 11 );
        else if ( line.startsWith( "City: " ) )
            tempCity = line.mid( 6 );
    }

    if ( tempLong.isEmpty() || tempLat.isEmpty() ) {
        kDebug() << "Method 1 did not work out for " << ip;
        getIPMethod2( ip );
        return;
    }

    m_city[ ip ] = tempCity;
    m_longitude[ ip ] = tempLong;
    m_latitude[ ip ] = tempLat;

    kDebug() << ip << tempCity << tempLong << tempLat;
    m_progress->progressBar()->setValue( m_city.count() );
    if ( m_city.count() == m_ips.count() )
        addIPsToWidget();
}

void MessageRouteWidget::slotReplyFinished2( QNetworkReply *reply )
{
    QString tempLat, tempLong, tempCity, ip;
    const QList<QByteArray> data = reply->readAll().split( '\n' );
    ip = reply->url().toString().mid( 47 );
    foreach( const QByteArray& line, data ) {
        if ( line.startsWith( "LAT: " ) )
            tempLat = line.mid( 15, line.length()-19 );
        else if ( line.startsWith( "LONG: " ) )
            tempLong = line.mid( 15, line.length()-19 );
        else if ( line.startsWith( "CITY: " ) )
            tempCity = line.mid( 15, line.length()-19 );
    }

    if ( tempCity != "MARINA DEL REY" && !tempLong.isEmpty() && !tempLat.isEmpty() ) {
        m_city[ ip ] = tempCity;
        m_longitude[ ip ] = tempLong;
        m_latitude[ ip ] = tempLat;

        kDebug() << ip << tempCity << tempLong << tempLat;
    } else {
        kDebug() << "Method 2 gave invalid result. deleting " << ip;
        int stepsBeforeFinished =  m_ips.count() -  m_progress->progressBar()->value();

        m_ips.removeAll( ip );
        m_progress->progressBar()->setMaximum( m_ips.count() );
        m_progress->progressBar()->setValue( m_ips.count() - stepsBeforeFinished );
    }

    m_progress->progressBar()->setValue( m_city.count() );
    if ( m_city.count() == m_ips.count() )
        addIPsToWidget();
}

void MessageRouteWidget::addIPsToWidget()
{
    // copied from digikam...
    QDomDocument       kmlDocument;
    QDomImplementation impl;
    QDomProcessingInstruction instr = kmlDocument.createProcessingInstruction( "xml","version=\"1.0\" encoding=\"UTF-8\"" );
    kmlDocument.appendChild( instr );
    QDomElement kmlRoot = kmlDocument.createElementNS( "http://earth.google.com/kml/2.1","kml" );
    kmlDocument.appendChild( kmlRoot );

    QDomElement kmlAlbum = addKmlElement( kmlDocument, kmlRoot, "Document" );
    QDomElement kmlName = addKmlTextElement( kmlDocument, kmlAlbum, "name", "Geolocation" );

    int i=m_ips.count();
    QString coordinates;
    foreach( const QString& ip, m_ips ) {
        if ( m_longitude[ip].isEmpty() ) {
            --i;
            continue;
        }
        QDomElement kmlPlacemark = addKmlElement( kmlDocument, kmlAlbum, "Placemark" );
        addKmlTextElement( kmlDocument, kmlPlacemark, "name",
                           QString( "%1.%2 (%3)" ).arg( QString::number( i ), m_city[ip], ip ) );

        QDomElement kmlGeometry  = addKmlElement( kmlDocument, kmlPlacemark, "Point" );
        addKmlTextElement( kmlDocument, kmlGeometry, "coordinates",
                           QString( "%1,%2" ).arg( m_longitude[ip], m_latitude[ip] ) );
        addKmlTextElement( kmlDocument, kmlGeometry, "altitudeMode", "clampToGround" );
        addKmlTextElement( kmlDocument, kmlGeometry, "extrude", "1" );

        coordinates += QString( "%1,%2 " ).arg( m_longitude[ip], m_latitude[ip] );
        --i;
    }

    QDomElement kmlElement = addKmlElement( kmlDocument, kmlAlbum, "Placemark" );
    addKmlTextElement( kmlDocument, kmlElement, "styleUrl", "#geolocation" );
    QDomElement kmlGeometry  = addKmlElement( kmlDocument, kmlElement, "LineString" );
    addKmlTextElement( kmlDocument, kmlGeometry, "coordinates", coordinates );

    kmlElement = addKmlElement( kmlDocument, kmlAlbum, "StyleMap" );
    kmlElement.setAttribute( "id", "geolocation" );
    QDomElement kmlLine = addKmlElement( kmlDocument, kmlElement, "Pair" );
    addKmlTextElement( kmlDocument, kmlLine, "key", "normal" );
    addKmlTextElement( kmlDocument, kmlLine, "styleUrl", "#normalState" );

    kmlElement = addKmlElement( kmlDocument, kmlAlbum, "Style" );
    kmlElement.setAttribute( "id", "normalState" );
    kmlLine = addKmlElement( kmlDocument, kmlElement, "LineStyle" );
    addKmlTextElement( kmlDocument, kmlLine, "width", "5" );
    addKmlTextElement( kmlDocument, kmlLine, "color", "7F0000FF" );

    /*
    QDomElement kmlTimeStamp = addKmlElement(kmlDocument, kmlPlacemark, "TimeStamp");
    addKmlTextElement(kmlDocument, kmlTimeStamp, "when",
                            (*it).dateTime.toString("yyyy-MM-ddThh:mm:ssZ"));
    */

    m_marble->setHome( m_longitude[m_ips.last()].toFloat(), m_latitude[m_ips.last()].toFloat() );
    m_marble->centerOn( m_longitude[m_ips.last()].toFloat(), m_latitude[m_ips.last()].toFloat() );
    kDebug() << kmlDocument.toString();
    m_marble->addPlaceMarkData( kmlDocument.toString() );
}

// copied from digikam...
QDomElement MessageRouteWidget::addKmlElement( QDomDocument &kmlDocument, QDomElement &target, const QString& tag )
{
    QDomElement kmlElement = kmlDocument.createElement( tag );
    target.appendChild( kmlElement );
    return kmlElement;
}

// copied from digikam...
QDomElement MessageRouteWidget::addKmlTextElement( QDomDocument &kmlDocument, QDomElement &target,
        const QString& tag, const QString& text )
{
    QDomElement kmlElement  = kmlDocument.createElement( tag );
    target.appendChild( kmlElement );
    QDomText kmlTextElement = kmlDocument.createTextNode( text );
    kmlElement.appendChild( kmlTextElement );
    return kmlElement;
}


#include "messageroutewidget.moc"
